@@ -13,9 +13,9 @@ __all__ = ("StructureDataTranslator",)
 
 
 class StructureDataTranslator(AiidaEntityTranslator):
-    """Create OPTiMaDe "structures" attributes from an AiiDA StructureData Node
+    """Create OPTIMADE "structures" attributes from an AiiDA StructureData Node
 
-    Each OPTiMaDe field is a method in this class.
+    Each OPTIMADE field is a method in this class.
 
     NOTE: This class succeeds in *never* loading the actual AiiDA Node for optimization purposes.
     """
@@ -41,7 +41,7 @@ class StructureDataTranslator(AiidaEntityTranslator):
             self.__sites = self._get_unique_node_property("attributes.sites")
         return self.__sites
 
-    # Helper methods to calculate OPTiMaDe fields
+    # Helper methods to calculate OPTIMADE fields
     def get_symbols_set(self):
         """Copy of aiida.orm.StructureData:get_symbols_set()"""
         return set(
@@ -130,7 +130,6 @@ class StructureDataTranslator(AiidaEntityTranslator):
     # Start creating fields
     def elements(self) -> List[str]:
         """Names of elements found in the structure as a list of strings, in alphabetical order."""
-
         attribute = "elements"
 
         if attribute in self.new_attributes:
@@ -142,13 +141,12 @@ class StructureDataTranslator(AiidaEntityTranslator):
         if "X" in res:
             res.remove("X")
 
-        # Finally, save OPTiMaDe attribute for later storage in extras for AiiDA Node and return value
+        # Finally, save OPTIMADE attribute for later storage in extras for AiiDA Node and return value
         self.new_attributes[attribute] = res
         return res
 
     def nelements(self) -> int:
         """Number of different elements in the structure as an integer."""
-
         attribute = "nelements"
 
         if attribute in self.new_attributes:
@@ -156,13 +154,12 @@ class StructureDataTranslator(AiidaEntityTranslator):
 
         res = len(self.elements())
 
-        # Finally, save OPTiMaDe attribute for later storage in extras for AiiDA Node and return value
+        # Finally, save OPTIMADE attribute for later storage in extras for AiiDA Node and return value
         self.new_attributes[attribute] = res
         return res
 
     def elements_ratios(self) -> List[float]:
         """Relative proportions of different elements in the structure."""
-
         attribute = "elements_ratios"
 
         if attribute in self.new_attributes:
@@ -173,13 +170,12 @@ class StructureDataTranslator(AiidaEntityTranslator):
         total_weight = sum(ratios.values())
         res = [ratios[symbol] / total_weight for symbol in self.elements()]
 
-        # Finally, save OPTiMaDe attribute for later storage in extras for AiiDA Node and return value
+        # Finally, save OPTIMADE attribute for later storage in extras for AiiDA Node and return value
         self.new_attributes[attribute] = str(res)
         return res
 
     def chemical_formula_descriptive(self) -> str:
         """The chemical formula for a structure as a string in a form chosen by the API implementation."""
-
         attribute = "chemical_formula_descriptive"
 
         if attribute in self.new_attributes:
@@ -187,7 +183,7 @@ class StructureDataTranslator(AiidaEntityTranslator):
 
         res = self.get_formula()
 
-        # Finally, save OPTiMaDe attribute for later storage in extras for AiiDA Node and return value
+        # Finally, save OPTIMADE attribute for later storage in extras for AiiDA Node and return value
         self.new_attributes[attribute] = res
         return res
 
@@ -201,7 +197,6 @@ class StructureDataTranslator(AiidaEntityTranslator):
         that within reasonable approximation indicate the correct chemical proportions.
         The precise details of how to perform the rounding is chosen by the API implementation.
         """
-
         attribute = "chemical_formula_reduced"
 
         if attribute in self.new_attributes:
@@ -226,7 +221,7 @@ class StructureDataTranslator(AiidaEntityTranslator):
                     occupation[symbol] = rounded_weight
         res = "".join([f"{symbol}{occupation[symbol]}" for symbol in self.elements()])
 
-        # Finally, save OPTiMaDe attribute for later storage in extras for AiiDA Node and return value
+        # Finally, save OPTIMADE attribute for later storage in extras for AiiDA Node and return value
         self.new_attributes[attribute] = res
         return res
 
@@ -241,7 +236,6 @@ class StructureDataTranslator(AiidaEntityTranslator):
 
         NOTE: This will always be equal to chemical_formula_descriptive if it should not be handled as unset.
         """
-
         attribute = "chemical_formula_hill"
 
         if attribute in self.new_attributes:
@@ -252,7 +246,7 @@ class StructureDataTranslator(AiidaEntityTranslator):
         else:
             res = self.get_formula(mode="hill")
 
-        # Finally, save OPTiMaDe attribute for later storage in extras for AiiDA Node and return value
+        # Finally, save OPTIMADE attribute for later storage in extras for AiiDA Node and return value
         self.new_attributes[attribute] = res
         return res
 
@@ -297,7 +291,7 @@ class StructureDataTranslator(AiidaEntityTranslator):
             [f"{map_anonymous[symbol]}{occupation[symbol]}" for symbol in elements]
         )
 
-        # Finally, save OPTiMaDe attribute for later storage in extras for AiiDA Node and return value
+        # Finally, save OPTIMADE attribute for later storage in extras for AiiDA Node and return value
         self.new_attributes[attribute] = res
         return res
 
@@ -309,7 +303,6 @@ class StructureDataTranslator(AiidaEntityTranslator):
         or non-periodic (value 0). Note: the elements in this list each refer to the direction
         of the corresponding entry in property lattice_vectors and not the Cartesian x, y, z directions.
         """
-
         attribute = "dimension_types"
 
         if attribute in self.new_attributes:
@@ -323,13 +316,33 @@ class StructureDataTranslator(AiidaEntityTranslator):
             )
         ]
 
-        # Finally, save OPTiMaDe attribute for later storage in extras for AiiDA Node and return value
+        # Finally, save OPTIMADE attribute for later storage in extras for AiiDA Node and return value
+        self.new_attributes[attribute] = res
+        return res
+
+    def nperiodic_dimensions(self) -> int:
+        """Number of periodic dimensions."""
+        attribute = "nperiodic_dimensions"
+
+        if attribute in self.new_attributes:
+            return self.new_attributes[attribute]
+
+        res = sum(
+            [
+                int(value)
+                for value in (
+                    self._get_unique_node_property(f"attributes.pbc{i + 1}")
+                    for i in range(3)
+                )
+            ]
+        )
+
+        # Finally, save OPTIMADE attribute for later storage in extras for AiiDA Node and return value
         self.new_attributes[attribute] = res
         return res
 
     def lattice_vectors(self) -> List[List[float]]:
         """The three lattice vectors in Cartesian coordinates, in ångström (Å)."""
-
         attribute = "lattice_vectors"
 
         if attribute in self.new_attributes:
@@ -339,7 +352,7 @@ class StructureDataTranslator(AiidaEntityTranslator):
             self._get_unique_node_property("attributes.cell")
         )
 
-        # Finally, save OPTiMaDe attribute for later storage in extras for AiiDA Node and return value
+        # Finally, save OPTIMADE attribute for later storage in extras for AiiDA Node and return value
         self.new_attributes[attribute] = res
         return res
 
@@ -349,7 +362,6 @@ class StructureDataTranslator(AiidaEntityTranslator):
         A site is an atom, a site potentially occupied by an atom,
         or a placeholder for a virtual mixture of atoms (e.g., in a virtual crystal approximation).
         """
-
         attribute = "cartesian_site_positions"
 
         if attribute in self.new_attributes:
@@ -358,13 +370,12 @@ class StructureDataTranslator(AiidaEntityTranslator):
         sites = [list(site["position"]) for site in self._sites]
         res = self.check_floating_round_errors(sites)
 
-        # Finally, save OPTiMaDe attribute for later storage in extras for AiiDA Node and return value
+        # Finally, save OPTIMADE attribute for later storage in extras for AiiDA Node and return value
         self.new_attributes[attribute] = res
         return res
 
     def nsites(self) -> int:
         """An integer specifying the length of the cartesian_site_positions property."""
-
         attribute = "nsites"
 
         if attribute in self.new_attributes:
@@ -372,7 +383,7 @@ class StructureDataTranslator(AiidaEntityTranslator):
 
         res = len(self.cartesian_site_positions())
 
-        # Finally, save OPTiMaDe attribute for later storage in extras for AiiDA Node and return value
+        # Finally, save OPTIMADE attribute for later storage in extras for AiiDA Node and return value
         self.new_attributes[attribute] = res
         return res
 
@@ -382,7 +393,6 @@ class StructureDataTranslator(AiidaEntityTranslator):
         (Where values for sites are specified with the same order of the property
         cartesian_site_positions). The properties of the species are found in the property species.
         """
-
         attribute = "species_at_sites"
 
         if attribute in self.new_attributes:
@@ -390,7 +400,7 @@ class StructureDataTranslator(AiidaEntityTranslator):
 
         res = [site["kind_name"] for site in self._sites]
 
-        # Finally, save OPTiMaDe attribute for later storage in extras for AiiDA Node and return value
+        # Finally, save OPTIMADE attribute for later storage in extras for AiiDA Node and return value
         self.new_attributes[attribute] = res
         return res
 
@@ -441,13 +451,12 @@ class StructureDataTranslator(AiidaEntityTranslator):
 
             res.append(species)
 
-        # Finally, save OPTiMaDe attribute for later storage in extras for AiiDA Node and return value
+        # Finally, save OPTIMADE attribute for later storage in extras for AiiDA Node and return value
         self.new_attributes[attribute] = res
         return res
 
     # def assemblies(self) -> List[dict]:
     #     """A description of groups of sites that are statistically correlated."""
-
     #     attribute = "assemblies"
 
     #     if attribute in self.new_attributes:
@@ -455,7 +464,7 @@ class StructureDataTranslator(AiidaEntityTranslator):
 
     #     res = []
 
-    #     # Finally, save OPTiMaDe attribute for later storage in extras for AiiDA Node and return value
+    #     # Finally, save OPTIMADE attribute for later storage in extras for AiiDA Node and return value
     #     self.new_attributes[attribute] = res
     #     return res
 
@@ -464,7 +473,6 @@ class StructureDataTranslator(AiidaEntityTranslator):
 
         SHOULD be absent if there are no partial occupancies
         """
-
         attribute = "structure_features"
 
         if attribute in self.new_attributes:
@@ -505,6 +513,6 @@ class StructureDataTranslator(AiidaEntityTranslator):
         # if self.assemblies():
         #     res.append("assemblies")
 
-        # Finally, save OPTiMaDe attribute for later storage in extras for AiiDA Node and return value
+        # Finally, save OPTIMADE attribute for later storage in extras for AiiDA Node and return value
         self.new_attributes[attribute] = res
         return res
